﻿--[[
WarlockReminder v1.70.3
Date: 2012-02-15T10:35:45Z
Author: cklausi
Original Author of WarlockReminder: Vux of Dragonblight-US 
]]--

WarlockReminder = LibStub("AceAddon-3.0"):NewAddon("WarlockReminder", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("WarlockReminder")
local media = LibStub("LibSharedMedia-3.0")

-- Default Values
local defaults = {
    profile = {
		--Reminder
		showArmor = true,
		showSL = true,
		showGOS = true,
		showFlask = true,
		showPet = true,
		showFood = true,
		showDI = true,
		showDemonicRebirth = true,
		-- Pets
		petAffliction = "felhunter",
		petDemonology = "felguard",
		petDestruction = "imp",
		-- DS
		DrainSoulEnable = true,
		DrainSoulSound = "Quaddamage",
		drainSoulPercent = 20 / 100,
		drainSoulHealth = 1000,
		-- Output
		showTimeleft = true,
		timeleftFontsize = 11,
		timeleftFont = "Friz Quadrata TT",
		-- Anchor
		wlrAnchorTop = (UIParent:GetHeight()/2)+60, 
		wlrAnchorLeft = (UIParent:GetWidth()/2)-60,
		wlrAnchorAlpha = 1,
		wlrAnchorSize = 1,
		-- MoltenCore
		MoltenCoreEnable = true,
		MoltenCoreSound = "Clearcasting",
		-- Shadow Trance
		ShadowTranceEnable = true,
		ShadowTranceSound = "Clearcasting",
		-- Demonic Fury
		DemonicFuryEnable = true,
		-- Burning Embers
		BurningEmbersEnable = true,
		-- Soul Shards
		SoulShardsEnable = true,
		-- Custom
		showCustom = true,
		CustomSound = "Clearcasting",
		showCustomTimeleft = true,
		customTimeleftFontsize = 12,
		customTimeleftFont = "Friz Quadrata TT",
		wlrCustomAnchorTop = (UIParent:GetHeight()/2)+60,
		wlrCustomAnchorLeft = (UIParent:GetWidth()/2)+60,
		wlrCustomAnchorAlpha = 1,
		wlrCustomAnchorSize = 1,
		CustomSpells = {}
    },
}
-- END Default Values

-- Spell Ids
local SoulLink = 108415
local DrainSoul = 1120
local SoulFire = 6353
local DemonicRebirth = 108559
local GrimoireOfSacrifice = 108503
local DemonicFury = 104315
local BurningEmbers = 108647
local SoulShards = 117198

-- Flask
-- Wotlk
local FlaskOfFrostwyrm = 53755
local SpellpowerElixir = 33721
-- Cata
local ElixirOfTheMaster = 79635
local ElixirOfTheMightySpeed = 79632
local ElixirOfImpossibleAccuracy = 79481
local ElixirOfTheCobra = 79477
local FlaskOfTheDraconicMind = 79470
-- Mop
local FlaskOfTheWarmSun = 114771

local CurrentFlask = FlaskOfTheWarmSun

-- Well Fed
local WellFed = 57399 -- there are so many different buffs - using textures
local  _, _, WellFedIcon = GetSpellInfo(WellFed)

-- Pets
local Pets = {
	["imp"] = 688,
	["voidwalker"] = 697,
	["succubus"] = 712,
	["felhunter"] = 691,
	["felguard"] = 30146,
}
local petName = ""

-- Procs
local MoltenCore = 122351
local ShadowTrance = 17941

-- Dark Intent
local DarkIntent = 109773

-- Custom
local FelSpark = 89937

-- Variables
local wlrDB
local ReminderHide = false
local timeLeft = 0
local timeelapsed = 0
local CustomPlayed = false
local CustomShow = false
local DrainSoulPlayed = false
local ReminderPlayed = false
local DemonicRebirthPlayed = false
local _, englishClass = UnitClass("player")
local SelectedSpells = {}
local ConfigMode = false
local Spec = GetSpecialization(false, false, GetActiveSpecGroup(false, false))

-- Frames
local ReminderFrame
local ReminderIcons
local wlrcustomanchor
local wlrcustomframe
local wlrcustomtexture
local wlrUnlockDialog

-- Addon start
function WarlockReminder:OnInitialize()
	if(englishClass == "WARLOCK") then
		self.db = LibStub("AceDB-3.0"):New("WarlockReminderDB", defaults, "Default")
		
		self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
		self.db.RegisterCallback(self, "OnProfileReset", "OnProfileReset")
		self.db.RegisterCallback(self, "OnNewProfile", "OnNewProfile")
		
		wlrDB = self.db.profile
		
		self:CreateFrames()
		
		self:ReminderFrameSetup()
		
		self:SetUpOptions()
	end
end

function WarlockReminder:CreateDefaultTables()
	wlrDB.CustomSpells = {
		[89937] = GetSpellInfo(89937),
		[70840] = GetSpellInfo(70840),
	}
end

function WarlockReminder:OnEnable()
	if(englishClass == "WARLOCK") then
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
		
		media:Register("sound", "Quaddamage", "Interface\\AddOns\\WarlockReminder\\Sounds\\quaddamage.ogg")
		media:Register("sound", "Clearcasting", "Sound\\Spells\\Clearcasting_Impact_Chest.wav")
		
		-- CustomReminder Position
		if (not wlrDB.wlrCustomAnchorLeft and not wlrDB.wlrCustomAnchorTop) then
			wlrcustomanchor:Show()
			wlrDB.wlrCustomAnchorLeft(wlrcustomanchor:GetLeft())
			wlrDB.wlrCustomAnchorTop(wlrcustomanchor:GetTop())
		else
			self:CustomSetup()
		end
	end
end
-- END Addon start


-- Reminderfactory
function WarlockReminder:NewReminder(SpellIds, Alpha, Size, Left, Top)
	local f = CreateFrame("Frame", nil, UIParent)
	f:SetToplevel(1)
	f:SetPoint("CENTER",0,0)
	f:SetAlpha(Alpha)
	f:SetFrameStrata("BACKGROUND")
	f:SetWidth(Size*64) 
	f:SetHeight(#SpellIds*f:GetWidth())
	
	-- Move
	f:SetScript("OnMouseDown", 
		function(self)
			if(self:IsMovable() == 1) then
				self:StartMoving()
			end
		end
	)
	f:SetScript("OnEvent",function(self,event,...) self[event](self,event,...) end)
	
	local i = {}
	local j = 0
	for _, value in pairs(SpellIds) do
		i[value] = CreateFrame("Frame", nil, f)
		i[value]:SetAlpha(Alpha)
		i[value]:SetFrameStrata("BACKGROUND")
		i[value]:SetWidth(64*Size)
		i[value]:SetHeight(64*Size)
		i[value]:SetPoint("TOP", f, "TOP", 0, -(j*64*Size))
		
		local t = i[value]:CreateTexture(nil, "BACKGROUND")
		if(value == "pets") then t:SetTexture(GetSpellTexture(Pets["imp"]))
		else t:SetTexture(GetSpellTexture(value)) end
		t:SetAllPoints(i[value])
		i[value].texture = t
		
		local text = i[value]:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
		text:SetFont(media:Fetch("font", wlrDB.timeleftFont), wlrDB.timeleftFontsize, "OUTLINE")
		text:SetPoint("BOTTOMRIGHT",-1,3)
		i[value].text = text
		
		i[value]:Show()
		j = j+1
	end
			
	f:SetScript("OnUpdate", function(self, elapsed)
			WarlockReminder:OnUpdate(self, elapsed)
		end
	)
	
	f:Show()
	
	return f, i
end

function WarlockReminder:SortReminder(Icons, Reminder, Alpha, Size, Left, Top)
	local i = 0
	for index, value in pairs(Icons) do
		if(Icons[index]:IsShown()) then
			ReminderFrame:SetWidth(Size*64) 
			ReminderFrame:SetHeight((i+1)*ReminderFrame:GetWidth())
			ReminderFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", Left, Top)
			Icons[index]:SetWidth(64*Size)
			Icons[index]:SetHeight(64*Size)
			Icons[index]:SetPoint("TOP", Reminder, "TOP", 0, -(i*64*Size))
			Icons[index].text:SetFont(media:Fetch("font", wlrDB.timeleftFont), wlrDB.timeleftFontsize, "OUTLINE")
			if(index ~= BurningEmbers and index ~= SoulShards and index ~= DemonicFury) then Icons[index]:SetAlpha(Alpha) end
			i = i+1
		end
	end
	
	Reminder:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
end


local function round(number, decimals)
    return (("%%.%df"):format(decimals)):format(number)
end

local function GetTimeLeft(expirationTime)
	if(expirationTime == nil or tonumber(expirationTime) == nil or wlrDB.showTimeleft == false) then 
		timeleft = "" 
	else 
		timeleft = (-1*(GetTime()-expirationTime))
	end
	
	return timeleft
end

-- Check if the unit has a buff
local function UnitHasBuff(unit, buff)
    for i=1, 40 do
		local name, _, iconTexture, count, _, _, expirationTime, source, _ = UnitBuff(unit, i)
        if ( name == GetSpellInfo(buff) ) then
			return true, iconTexture, count, expirationTime, source
        end
    end
	return false
end

-- EVENTS
function WarlockReminder:PLAYER_TARGET_CHANGED()
	if(ConfigMode == false) then
		DrainSoulPlayed = false
	end
end

function WarlockReminder:ACTIVE_TALENT_GROUP_CHANGED()
	Spec = GetSpecialization(false, false, GetActiveSpecGroup(false, false))
end

function WarlockReminder:PLAYER_SPECIALIZATION_CHANGED()
	Spec = GetSpecialization(false, false, GetActiveSpecGroup(false, false))
end
-- END EVENTS

-- Warlock reminder
function WarlockReminder:Reminder()

	--local test = UnitPower("player", SPELL_POWER_DEMONIC_FURY)
	--self:Print(test)

	-- Bufffood
	if(wlrDB.showFood and
		UnitLevel("target") < 0 and
		UnitHealth("target") * 100 / UnitHealthMax("target") > 98 and
		select(2, UnitHasBuff("player", WellFed)) ~= WellFedIcon
	) then
		ReminderIcons[WellFed]:Show()
	else
		ReminderIcons[WellFed]:Hide()
	end

	-- Flask & Elixir
	if(wlrDB.showFlask and
		UnitLevel("target") < 0 and
		UnitHealth("target") * 100 / UnitHealthMax("target") > 98 and
		-- Wotlk
		not UnitHasBuff("player", FlaskOfFrostwyrm) and
		not UnitHasBuff("player", SpellpowerElixir) and
		-- Cata
		not UnitHasBuff("player", ElixirOfTheMaster) and
		not UnitHasBuff("player", ElixirOfTheMightySpeed) and
		not UnitHasBuff("player", ElixirOfImpossibleAccuracy) and
		not UnitHasBuff("player", ElixirOfTheCobra) and
		not UnitHasBuff("player", FlaskOfTheDraconicMind) and
		-- Mop
		not UnitHasBuff("player", FlaskOfTheWarmSun)
	) then
		ReminderIcons[CurrentFlask]:Show()
	else
		ReminderIcons[CurrentFlask]:Hide()
	end
	
	-- GrimoireOfSacrifice
	local _, _, _, _, selectedGOS, _ = GetTalentInfo(15)
	if(selectedGOS and
		wlrDB.showGOS and
		not IsMounted("player") and
		not UnitHasBuff("player", GrimoireOfSacrifice)
	) then
		ReminderIcons[GrimoireOfSacrifice]:Show()
	else
		ReminderIcons[GrimoireOfSacrifice]:Hide()
	end

	-- Soul Link
	local _, _, _, _, selectedSL, _ = GetTalentInfo(7)
	if(selectedSL and
		not selectedGOS and
		wlrDB.showSL and
		UnitExists("pet") and
		not IsMounted("player") and
		not UnitHasBuff("player", SoulLink)
	) then
		ReminderIcons[SoulLink]:Show()
	else
		ReminderIcons[SoulLink]:Hide()
	end
	
	-- DI Missing
	if( wlrDB.showDI and 
		not UnitHasBuff("player", DarkIntent) and
		not IsMounted("player")
	) then
		ReminderIcons[DarkIntent]:Show()
	else
		ReminderIcons[DarkIntent]:Hide()
	end

	-- Pet
	if(wlrDB.showPet and
		not selectedGOS and
		not UnitExists("pet") and
		not IsMounted("player")
	) then
		if Spec == 1 then ReminderIcons["pets"].texture:SetTexture(GetSpellTexture(Pets[wlrDB.petAffliction])) -- Affli
		elseif (Spec == 2) then ReminderIcons["pets"].texture:SetTexture(GetSpellTexture(Pets[wlrDB.petDemonology])) -- Demo
		else ReminderIcons["pets"].texture:SetTexture(GetSpellTexture(Pets[wlrDB.petDestruction])) end -- Destro + unknown
		ReminderIcons["pets"]:Show()
	else
		ReminderIcons["pets"]:Hide()
	end
	
	-- Demonic Rebirth
	if(wlrDB.showDemonicRebirth and 
		UnitHasBuff("player", DemonicRebirth)
	) then
		if(wlrDB.showTimeleft) then
			local _, _, _, expirationTime = UnitHasBuff("player", DemonicRebirth)
			local timeleft = GetTimeLeft(expirationTime)
			ReminderIcons[DemonicRebirth].text:SetText(tostring(round(timeleft, 1)) .. L["seconds"])
		else
			ReminderIcons[DemonicRebirth].text:SetText("")
		end
		ReminderIcons[DemonicRebirth]:Show()
	else
		ReminderIcons[DemonicRebirth]:Hide()
	end
	
	-- Drain Soul
	if(wlrDB.DrainSoulEnable and 
		UnitCanAttack("player", "target") and
		not UnitIsDead("target") and
		UnitHealth("target") > 0 and
		UnitHealth("target") * 100 / UnitHealthMax("target") <= wlrDB.drainSoulPercent * 100 and
		UnitHealthMax("target") >= tonumber(wlrDB.drainSoulHealth)
	) then
		ReminderIcons[DrainSoul]:Show()
		if(not DrainSoulPlayed) then
			PlaySoundFile(media:Fetch("sound", wlrDB.DrainSoulSound))
			DrainSoulPlayed = true
		end
	else
		DrainSoulPlayed = false
		ReminderIcons[DrainSoul]:Hide()
	end
	
	-- Moltencore
	if(wlrDB.MoltenCoreEnable and 
		(UnitHasBuff("player", MoltenCore) or
		UnitHasBuff("player", MoltenCore) or
		UnitHasBuff("player", MoltenCore))
	) then
		local _, _, count, expirationTime = UnitHasBuff("player", MoltenCore)
		if(count == nil) then _, _, count, expirationTime = UnitHasBuff("player", MoltenCore) end
		if(count == nil) then _, _, count, expirationTime = UnitHasBuff("player", MoltenCore) end
		local timeleft = 0
		if(wlrDB.showTimeleft) then
			timeleft = tostring(round(GetTimeLeft(expirationTime), 1)) .. L["seconds"]
		else
			timeleft = ""
		end
		local text = tostring(count).."x  ".. timeleft
		ReminderIcons[MoltenCore].text:SetText(text)
		ReminderIcons[MoltenCore]:Show()
		if(not MoltenCorePlayed) then
			PlaySoundFile(media:Fetch("sound", wlrDB.MoltenCoreSound))
			MoltenCorePlayed = true
		end
	else
		ReminderIcons[MoltenCore]:Hide()
		MoltenCorePlayed = false
	end
	
	-- Shadowtrance
	if(wlrDB.ShadowTranceEnable and 
		UnitHasBuff("player", ShadowTrance)
	) then
		if(wlrDB.showTimeleft) then
			local _, _, _, expirationTime = UnitHasBuff("player", ShadowTrance)
			local timeleft = GetTimeLeft(expirationTime)
			ReminderIcons[ShadowTrance].text:SetText(tostring(round(timeleft, 1)) .. L["seconds"])
		else
			ReminderIcons[ShadowTrance].text:SetText("")
		end
		ReminderIcons[ShadowTrance]:Show()
		if(not ShadowTrancePlayed) then
			PlaySoundFile(media:Fetch("sound", wlrDB.ShadowTranceSound))
			ShadowTrancePlayed = true
		end
	else
		ReminderIcons[ShadowTrance]:Hide()
		ShadowTrancePlayed = false
	end
	
	-- Soul Shards
	if(wlrDB.SoulShardsEnable and
		Spec == 1
	) then
		local Percent, Value = self:Power(SPELL_POWER_SOUL_SHARDS)
		ReminderIcons[SoulShards].text:SetText(tostring(Value))
		ReminderIcons[SoulShards]:SetAlpha(Percent)
		ReminderIcons[SoulShards]:Show()
	else
		ReminderIcons[SoulShards]:Hide()
	end
	
	-- Demonic Fury
	if(wlrDB.DemonicFuryEnable and
		Spec == 2
	) then
		local Percent, Value = self:Power(SPELL_POWER_DEMONIC_FURY)
		ReminderIcons[DemonicFury].text:SetText(tostring(Value))
		ReminderIcons[DemonicFury]:SetAlpha(Percent)
		ReminderIcons[DemonicFury]:Show()
	else
		ReminderIcons[DemonicFury]:Hide()
	end
	
	-- Burning Embers
	if(wlrDB.BurningEmbersEnable and
		Spec == 3
	) then
		local Percent, Value = self:Power(SPELL_POWER_BURNING_EMBERS)
		ReminderIcons[BurningEmbers].text:SetText(tostring(Value))
		ReminderIcons[BurningEmbers]:SetAlpha(Percent)
		ReminderIcons[BurningEmbers]:Show()
	else
		ReminderIcons[BurningEmbers]:Hide()
	end
		
	-- Rearrange Icons
	self:SortReminder(ReminderIcons, ReminderFrame, wlrDB.wlrAnchorAlpha, wlrDB.wlrAnchorSize, wlrDB.wlrAnchorLeft, wlrDB.wlrAnchorTop)
	
end

function WarlockReminder:Power(PowerType)
	local Value = UnitPower("player", PowerType)
	local Max = UnitPowerMax("player", PowerType)
	local Percent = 0
	if(Max > 0) then Percent = Value / Max end
	if(Percent < 0.2) then Percent = 0.2 end
	
	return Percent, Value
end


-- OnUpdate
function WarlockReminder:OnUpdate(self, elapsed)
	timeelapsed = timeelapsed + elapsed
	if(timeelapsed >= 0.05) then
     
		if(ConfigMode == false) then
			WarlockReminder:Reminder()
			WarlockReminder:CustomReminder()
		end
      timeelapsed = 0
    end
end

-- Customreminder spells
function WarlockReminder:GetCustomSpells()
	return wlrDB.CustomSpells
end

function WarlockReminder:GetCurrentSpells(info, key)
	return SelectedSpells[key];
end

function WarlockReminder:SetCurrentSpells(info, key, value)
	if(not value) then
		value = nil;
	end
	SelectedSpells[key] = value;
end

function WarlockReminder:AddCustomSpell(info, value)
	local found = false
	-- Spell already exists?
	for index, spell in pairs(wlrDB.CustomSpells) do
		if(index == tonumber(value)) then
			found = true
			break
		end
	end
	-- Spell not in list -> add spell
	if(found == false) then
		wlrDB.CustomSpells[tonumber(value)] = GetSpellInfo(value)
	end
end

function WarlockReminder:RemoveCustomSpell(info, value)
	if wlrDB.CustomSpells ~= nil then
		for index, _ in pairs(SelectedSpells) do
			wlrDB.CustomSpells[index] = nil
			table.remove(wlrDB.CustomSpells, index)
		end
	end
	SelectedSpells = {}
end

-- Custom reminder
function WarlockReminder:CustomReminder()
	CustomShow = false
	if(wlrDB.showCustom) then
		for spell, _ in pairs(wlrDB.CustomSpells) do
			if(UnitHasBuff("player", spell)) then
				self:WlrCustomFrame(true, spell, select(3, UnitHasBuff("player", spell)), select(4, UnitHasBuff("player", spell)))
				CustomShow = true
				if(CustomPlayed == false) then
					PlaySoundFile(media:Fetch("sound", wlrDB.CustomSound))
					CustomPlayed = true
				end
				break
			end
		end
		if(CustomShow == false) then 
			CustomPlayed = false
			self:WlrCustomFrame(false)
		end
	end
end

function WarlockReminder:CreateFrames()
	-- Create Frames
	ReminderFrame, ReminderIcons = self:NewReminder({SoulLink, GrimoireOfSacrifice, "pets", DarkIntent,
																	DrainSoul, MoltenCore, ShadowTrance, DemonicRebirth, DemonicFury, BurningEmbers, SoulShards,
																	CurrentFlask, WellFed},
													wlrDB.wlrAnchorAlpha, wlrDB.wlrAnchorSize, wlrDB.wlrAnchorLeft, wlrDB.wlrAnchorTop)
													
	ReminderFrame:SetScript("OnMouseUp",
		function(self) 
			if(self:IsMovable() == 1) then
				self:StopMovingOrSizing() 
				wlrDB.wlrAnchorLeft = self:GetLeft()
				wlrDB.wlrAnchorTop = self:GetTop()  
			end
		end
	)
	self:SortReminder(ReminderIcons, ReminderFrame, wlrDB.wlrAnchorAlpha, wlrDB.wlrAnchorSize, wlrDB.wlrAnchorLeft, wlrDB.wlrAnchorTop)
	
	CustomFrame, CustomIcons = self:NewReminder({FelSpark},
												wlrDB.wlrCustomAnchorAlpha, wlrDB.wlrCustomAnchorSize, wlrDB.wlrCustomAnchorLeft, wlrDB.wlrCustomAnchorTop)
	CustomFrame:SetScript("OnMouseUp",
		function(self) 
			if(self:IsMovable() == 1) then
				self:StopMovingOrSizing() 
				wlrDB.wlrCustomAnchorLeft = self:GetLeft()
				wlrDB.wlrCustomAnchorTop = self:GetTop()  
			end
		end
	)
	
	-- Set up customanchor
	wlrcustomanchor = CreateFrame("Frame", nil, UIParent), {}
	wlrcustomanchor:SetWidth(120)
	wlrcustomanchor:SetHeight(24)
	wlrcustomanchor:SetBackdrop({ bgFile = media:Fetch("background", "Blizzard Tooltip"), edgeFile = media:Fetch("border", "Blizzard Tooltip"), tile = true, tileSize = 8, edgeSize = 12, insets = { left = 2, right = 2, top = 2, bottom = 2 } })
	wlrcustomanchor:SetBackdropColor(0.1, 0.1, 0.2, 1)
	wlrcustomanchor:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
	wlrcustomanchor:SetMovable(1)
	wlrcustomanchor:EnableMouse(1)
	wlrcustomanchor:SetToplevel(1)
	wlrcustomanchor:SetClampedToScreen(1)
	wlrcustomanchor:SetPoint("CENTER")
	wlrcustomanchor:Hide()

	wlrcustomanchor.text = wlrcustomanchor:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	wlrcustomanchor.text:SetText("Custom Reminder")
	wlrcustomanchor.text:SetPoint("LEFT", 6, 0)

	wlrcustomanchor.close = CreateFrame("Button", nil, wlrcustomanchor,"UIPanelCloseButton")
	wlrcustomanchor.close:SetWidth(24)
	wlrcustomanchor.close:SetHeight(24)
	wlrcustomanchor.close:SetPoint("RIGHT")
	
	-- Set up customframe
	wlrcustomframe = CreateFrame("Frame", nil, UIParent), {}
	wlrcustomtexture = wlrcustomframe:CreateTexture()
	wlrcustomtexture:SetAllPoints(wlrcustomframe)
	wlrcustomframe.text = wlrcustomframe:CreateFontString(nil,"OVERLAY","GameFontHighlight")
	wlrcustomframe.text:SetFont(media:Fetch("font", "Friz Quadrata TT"), 20, "OUTLINE")
	wlrcustomframe.texttime = wlrcustomframe:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	wlrcustomframe.texttime:SetPoint("BOTTOMRIGHT",-4,-16)
	wlrcustomframe:Hide()
	
	-- Move customanchor
	wlrcustomanchor:SetScript("OnMouseDown",wlrcustomanchor.StartMoving)
	wlrcustomanchor:SetScript("OnMouseUp",
		function(self) 
			self:StopMovingOrSizing() 
			-- Set anchor
			wlrDB.wlrCustomAnchorLeft = self:GetLeft()
			wlrDB.wlrCustomAnchorTop = self:GetTop() 
			WarlockReminder:FramesToAnchor()
		end
	)
	wlrcustomanchor:SetScript("OnEvent",function(self,event,...) self[event](self,event,...) end)
end
-- END CreateFrames

function WarlockReminder:ResetAnchor()
	WarlockReminder:FramesToAnchor()
end

function WarlockReminder:ResetCustomAnchor()
	wlrcustomanchor:ClearAllPoints()
	wlrcustomanchor:SetPoint("CENTER",UIParent,"CENTER", 0, 0)
	wlrDB.wlrCustomAnchorLeft = wlrcustomanchor:GetLeft()
	wlrDB.wlrCustomAnchorTop = wlrcustomanchor:GetTop() 
	WarlockReminder:FramesToAnchor()
	wlrcustomanchor:Show()
end

function WarlockReminder:UnlockFrame()
	if not wlrUnlockDialog then
		local f = CreateFrame("Frame", "WlrUnlockDialog", UIParent)
		f:SetFrameStrata("DIALOG")
		f:SetToplevel(true)
		f:EnableMouse(true)
		f:SetClampedToScreen(true)
		f:SetWidth(360)
		f:SetHeight(110)
		f:SetBackdrop{
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background" ,
			edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
			tile = true,
			insets = {left = 11, right = 12, top = 12, bottom = 11},
			tileSize = 32,
			edgeSize = 32,
		}
		f:SetPoint("TOP", 0, -50)
		f:Hide()

		local tr = f:CreateTitleRegion()
		tr:SetAllPoints(f)

		local header = f:CreateTexture(nil, "ARTWORK")
		header:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
		header:SetWidth(256); header:SetHeight(64)
		header:SetPoint("TOP", 0, 12)

		local title = f:CreateFontString("ARTWORK")
		title:SetFontObject("GameFontNormal")
		title:SetPoint("TOP", header, "TOP", 0, -14)
		title:SetText("WarlockReminder")

		local desc = f:CreateFontString("ARTWORK")
		desc:SetFontObject("GameFontHighlight")
		desc:SetJustifyV("TOP")
		desc:SetJustifyH("LEFT")
		desc:SetPoint("TOPLEFT", 18, -32)
		desc:SetPoint("BOTTOMRIGHT", -18, 48)
		desc:SetText(L["Frames unlocked. Move them now and click Lock when you are done."])

		local lockBars = CreateFrame("CheckButton", "WlrUnlockDialogLock", f, "OptionsButtonTemplate")
		getglobal(lockBars:GetName() .. "Text"):SetText(L["Lock"])

		lockBars:SetScript("OnClick", function(self)
			WarlockReminder:Lock()
			LibStub("AceConfigRegistry-3.0"):NotifyChange("WarlockReminder")
		end)

		--position buttons
		lockBars:SetPoint("BOTTOMRIGHT", -14, 14)

		wlrUnlockDialog = f
	end
	wlrUnlockDialog:Show()
end

function WarlockReminder:Lock()
	ConfigMode = false
	wlrcustomanchor:Hide()
	wlrcustomframe.texttime:SetText("")
	wlrUnlockDialog:Hide()
	self:CustomSetup()
	self:ReminderFrameSetup()
	self:Reminder()
end

function WarlockReminder:Unlock()
	ConfigMode = true
	wlrcustomanchor:Show()
	self:WlrCustomFrame(true, FelSpark, 0)
	wlrcustomframe.texttime:SetText("10.0s")
	self:ReminderFrameSetup()
	self:CustomSetup()
	self:UnlockFrame()
end

-- ReminderFrame Setup
function WarlockReminder:ReminderFrameSetup()
	if(ConfigMode == true) then 
		ReminderFrame:SetMovable(1)
		ReminderFrame:EnableMouse(1)
		ReminderIcons[DarkIntent]:Show()
		ReminderIcons["pets"]:Show()
		ReminderIcons[ShadowTrance]:Show()
		ReminderIcons[CurrentFlask]:Show()
		self:SortReminder(ReminderIcons, ReminderFrame, wlrDB.wlrAnchorAlpha, wlrDB.wlrAnchorSize, wlrDB.wlrAnchorLeft, wlrDB.wlrAnchorTop)
		
		CustomFrame:SetMovable(1)
		CustomFrame:EnableMouse(1)
	else 
		ReminderFrame:SetMovable(0)
		ReminderFrame:EnableMouse(0)
		
		CustomFrame:SetMovable(0)
		CustomFrame:EnableMouse(0)
	end
end

-- CustomReminder Setup
function WarlockReminder:CustomSetup()
	wlrcustomanchor:ClearAllPoints()
	
	-- Position
	wlrcustomanchor:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", wlrDB.wlrCustomAnchorLeft, wlrDB.wlrCustomAnchorTop)
	
	-- Alpha
	wlrcustomframe:SetAlpha(wlrDB.wlrCustomAnchorAlpha)
	
	-- Size
	wlrcustomframe:SetWidth(48*wlrDB.wlrCustomAnchorSize)
	wlrcustomframe:SetHeight(48*wlrDB.wlrCustomAnchorSize)
	
	-- Timeleft
	wlrcustomframe.texttime:SetFont(media:Fetch("font", wlrDB.customTimeleftFont), wlrDB.customTimeleftFontsize, "OUTLINE")
	wlrcustomframe.texttime:SetPoint("CENTER", 0, -1*(48*wlrDB.wlrCustomAnchorSize)-15)
	
	self:FramesToAnchor()
	
	if(ConfigMode == false) then self:CustomReminder() end
	
	-- Show/Hide
	if(wlrcustomanchor:IsShown()) then 
		wlrcustomframe:Show()
	else 
		wlrcustomframe:Hide() 
		wlrcustomanchor:Hide()
	end
	
end

-- Attach all frames to the anchor
function WarlockReminder:FramesToAnchor()
	wlrcustomframe:SetPoint("CENTER",wlrcustomanchor,0,-42) 
end

-- Show the customreminder frame
function WarlockReminder:WlrCustomFrame(show, texture, count, expirationTime)
	local timeleft
	
	if(texture) then wlrcustomtexture:SetTexture(GetSpellTexture(texture))
	else wlrcustomtexture:SetTexture(GetSpellTexture(89937)) end
	
	if(not count or count == 0) then count = "" end
	
	if(show) then
		if( expirationTime == nil or tonumber(expirationTime) == nil or wlrDB.showCustomTimeleft == false ) then timeleft = "" 
		else 
			timeleft = (-1*(GetTime()-expirationTime))
		end
		
		if(timeleft == "" or timeleft == nil) then wlrcustomframe.texttime:SetText("")
		else wlrcustomframe.texttime:SetText(tostring(round(timeleft, 1)) .. L["seconds"]) end
		
		wlrcustomframe.text:SetText(tostring(count))
		wlrcustomframe.text:SetPoint("BOTTOMRIGHT",-3,3)
		wlrcustomframe:Show()
	else 
		wlrcustomframe:Hide()
	end
end


local TextPositions = {
	[0] = L["Right"],
	[1] = L["Left"],
	[2] = L["Center"],
}

local PetChoice = {
	["felhunter"] = L["Felhunter"],
	["succubus"] = L["Succubus"],
	["imp"] = L["Imp"],
	["voidwalker"] = L["Voidwalker"],
	["felguard"] = L["Felguard"],
}

-- Options
local wlroptions = {
    name = "WarlockReminder",
    handler = WarlockReminder,
    type = "group",
	childGroups = "tree",
    args = {
    	config = {
			type = "execute",
			name = L["Configure"],
			desc = L["Open the options dialog of WarlockReminder"],
			func = function()
				InterfaceOptionsFrame_OpenToCategory(WarlockReminder.optionFrames.profile)
				InterfaceOptionsFrame_OpenToCategory(WarlockReminder.optionFrames.main)
			end,
			guiHidden = true,
		},
		configmode = {
			type = "execute",
			name = L["Config Mode"],
			desc = L["Config Mode"],
			func = "Unlock",
			guiHidden = true,
		},
		reminder = {
            type = "group",
			order = 0,
            name = L["Reminders"],
            args = {
            	reminderheader = {
            		type = "header",
					order = 9,
					name = L["Reminders"],
            	},
				darkintent = {
					type = "toggle",
					order = 15,
					name = L["Dark Intent"],
					desc = L["Dark Intent"],
					arg = "showDI",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				demonicrebirth = {
					type = "toggle",
					order = 17,
					name = L["Demonic Rebirth"],
					desc = L["Demonic Rebirth"],
					arg = "showDemonicRebirth",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				sl = {
					type = "toggle",
					order = 20,
					name = L["Soul Link"],
					desc = L["Show Soul Link reminder"],
					arg = "showSL",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				gos = {
					type = "toggle",
					order = 20,
					name = L["Grimoire of Sacrifice"],
					desc = L["Grimoire of Sacrifice"],
					arg = "showGOS",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				flask = {
					type = "toggle",
					order = 30,
					name = L["Flask and Elixir"],
					desc = L["Show Flask and Elixir reminder"],
					arg = "showFlask",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				food = {
					type = "toggle",
					order = 40,
					name = L["Well Fed"],
					desc = L["Show food reminder"],
					arg = "showFood",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				pet = {
					type = "toggle",
					order = 50,
					name = L["Pet"],
					desc = L["Show pet reminder"],
					arg = "showPet",
					get = "getProfileOption",
					set = "setProfileOption",
					width = "full",
				},
				afflictionpet = {
					type = "select",
					name = L["Affliction"],
					desc = L["Pet for Affliction"],
					order = 60,
					values = PetChoice,
					arg = "petAffliction",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				demonpet = {
					type = "select",
					name = L["Demonology"],
					desc = L["Pet for Demonology"],
					order = 70,
					values = PetChoice,
					arg = "petDemonology",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				destrupet = {
					type = "select",
					name = L["Destruction"],
					desc = L["Pet for Destruction"],
					order = 80,
					values = PetChoice,
					arg = "petDestruction",
					get = "getProfileOption",
					set = "setProfileOption",
				},
			},
		},
		combat = {
            type = "group",
			order = 2,
            name = L["Combat Reminder"],
            args = {
				combatdesc = {
					order = 0,
					name = L["Combat reminder description"],
					type = "description",
				},
				drainsoul = {
					type = "group",
					order = 1,
					name = L["Drain Soul"],
					args = {
						enable = {
				    		type = "toggle",
				    		order  = 1,
				    		name = L["Enable"],
				    		desc = L["Enable"],
				    		arg = "DrainSoulEnable",
				    		get = "getProfileOption",
							set = "setProfileOption",
							width = "full",
				    	},
						percent = {
							type = "range",
							name = L["Show reminder at"],
							desc = L["Show Drain Soul reminder at"],
							order = 10,
							min = 0.0,
							max = 1.0,
							step = 0.01,
							isPercent = true,
							arg = "drainSoulPercent",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						health = {
							type = "input",
							name = L["Minimum health"],
							desc = L["Minimum health to show the Drain Soul reminder"],
							order = 20,
							arg = "drainSoulHealth",
							get = "getProfileOption",
							set = "setProfileOption",
						},
						sound = {
							type = 'select',
							dialogControl = 'LSM30_Sound',
							name = L["Sound reminder"],
							desc = L["Play Drain Soul sound reminder"],
							order = 30,
							values = AceGUIWidgetLSMlists.sound,
							arg = "DrainSoulSound",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				}, -- END Drainsoul
				shadowtrance = {
					type = "group",
					order = 10,
					name = L["Shadow Trance"],
					args = {
						enable = {
				    		type = "toggle",
				    		order  = 1,
				    		name = L["Enable"],
				    		desc = L["Enable"],
				    		arg = "ShadowTranceEnable",
				    		get = "getProfileOption",
							set = "setProfileOption",
							width = "full",
				    	},
						sound = {
							type = 'select',
							name = L["Sound reminder"],
							desc = L["Play Shadow Trance sound reminder"],
							order = 10,
							dialogControl = 'LSM30_Sound',
							values = AceGUIWidgetLSMlists.sound,
							arg = "ShadowTranceSound",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				}, -- END Shadow Trance
				moltencore = {
					type = "group",
					order = 30,
					name = L["Molten Core"],
					args = {
						enable = {
				    		type = "toggle",
				    		order  = 1,
				    		name = L["Enable"],
				    		desc = L["Enable"],
				    		arg = "MoltenCoreEnable",
				    		get = "getProfileOption",
							set = "setProfileOption",
							width = "full",
				    	},
						sound = {
							type = 'select',
							name = L["Sound reminder"],
							desc = L["Play Molten Core sound reminder"],
							order = 10,
							dialogControl = 'LSM30_Sound',
							values = AceGUIWidgetLSMlists.sound,
							arg = "MoltenCoreSound",
							get = "getProfileOption",
							set = "setProfileOption",
						},
					},
				}, -- END Molten Core
				soulshards = {
					type = "group",
					order = 40,
					name = L["Soul Shards"],
					args = {
						enable = {
				    		type = "toggle",
				    		order  = 1,
				    		name = L["Enable"],
				    		desc = L["Enable"],
				    		arg = "SoulShardsEnable",
				    		get = "getProfileOption",
							set = "setProfileOption",
							width = "full",
				    	},
					},
				}, -- END Demonic Fury
				demonicfury = {
					type = "group",
					order = 50,
					name = L["Demonic Fury"],
					args = {
						enable = {
				    		type = "toggle",
				    		order  = 1,
				    		name = L["Enable"],
				    		desc = L["Enable"],
				    		arg = "DemonicFuryEnable",
				    		get = "getProfileOption",
							set = "setProfileOption",
							width = "full",
				    	},
					},
				}, -- END Demonic Fury
				burningembers = {
					type = "group",
					order = 60,
					name = L["Burning Embers"],
					args = {
						enable = {
				    		type = "toggle",
				    		order  = 1,
				    		name = L["Enable"],
				    		desc = L["Enable"],
				    		arg = "BurningEmbersEnable",
				    		get = "getProfileOption",
							set = "setProfileOption",
							width = "full",
				    	},
					},
				}, -- END Demonic Fury
			},
		},
		-- CustomReminder
		customreminder = {
			type = "group",
			order = 3,
			name = L["Custom Reminder"],
			args = {
				enable = {
					type = "toggle",
					order = 20,
					name = L["Show Custom Reminder"],
					desc = L["Show Custom Reminder"],
					arg = "showCustom",
					get = "getProfileOption",
					set = "setProfileOption",
					width = "full",
				},
				configmode = {
					type = "execute",
					name = L["Config Mode"],
					desc = L["Config Mode"],
					order = 30,
					func = "Unlock",
				},
				reset = {
					type = "execute",
					name = L["Reset position"],
					desc = L["Anchor Reset"],
					order = 40,
					func = "ResetCustomAnchor",
				},
				sound = {
					type = 'select',
					name = L["Sound reminder"],
					desc = L["Sound reminder"],
					order = 50,
					dialogControl = 'LSM30_Sound',
					values = AceGUIWidgetLSMlists.sound,
					arg = "CustomSound",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				alpha = {
					type = "range",

					name = L["Frame alpha"],
					desc = L["Frame alpha"],
					order = 60,
					min = 0.0,
					softMin = 0.25,
					max = 1.0,
					step = 0.05,
					isPercent = true,
					arg = "wlrCustomAnchorAlpha",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				size = {
					type = "range",
					name = L["Frame size"],
					desc = L["Frame size"],
					order = 70,
					min = 0.01,
					softMin = 0.25,
					max = 3,
					softMax = 2,
					step = 0.01,
					isPercent = true,
					arg = "wlrCustomAnchorSize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				customtimeleftheader = {
					type = "header",
					order = 80,
					name = L["Show time left"],
				},
				timeleft = {
					type = "toggle",
					order = 90,
					name = L["Show time left"],
					desc = L["Show time left"],
					arg = "showCustomTimeleft",
					get = "getProfileOption",
					set = "setProfileOption",
					width = "full",
				},
				font = {
					type = 'select',
					dialogControl = 'LSM30_Font',
					name = L["Font"],
					desc = L["Font"],
					values = AceGUIWidgetLSMlists.font,
					arg = "customTimeleftFont",
					get = "getProfileOption",
					set = "setProfileOption",
					order = 100,
				},
				fontsize = {
					type = "range",
					name = L["Font size"],
					desc = L["Font size"],
					order = 110,
					min = 0.00,
					softMin = 7,
					max = 32,
					step = 1,
					arg = "customTimeleftFontsize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				remindersetup = {
					type = "group",
					order = 1,
					name = L["Spell setup"],
					args = {
						addspell = {
							name = L["Add a Spell"],
							desc = L["Add a new spell to the list by entering the Spell-Id here."],
							type = "input",
							order = 10,
							get = false,
							set = "AddCustomSpell",
							width = "double",
						},
						selectspell = {
							name = "",
							desc = "",
							type = "multiselect",
							order = 20,
							get = "GetCurrentSpells",
							set = "SetCurrentSpells",
							values = "GetCustomSpells",
							width = "double",
						},
						removespell = {
							name = L["Remove Spell"],
							desc = L["Remove the selected spell from the list."],
							type = "execute",
							func = "RemoveCustomSpell",
							order = 30,
						},
						customdesc = {
							order = 100,
							name = L["Use spell-ids from wowhead.com's Urls, e.g. Devious Minds: http://www.wowhead.com/spell=70840 => Spell-Id: 70840"],
							type = "description",
						},
					},
				},
			},
		}, -- END CustomReminder
		output = {
            type = "group",
			order = 4,
            name = L["Output Frame"],
            args = {
				anchorheader = {
					type = "header",
					order = 20,
					name = L["WarlockReminder Frame"],
				},
				configmode = {
					type = "execute",
					name = L["Config Mode"],
					desc = L["Config Mode"],
					order = 30,
					func = "Unlock",
				},
				reset = {
					type = "execute",
					name = L["Reset position"],
					desc = L["Anchor Reset"],
					order = 40,
					func = "ResetAnchor",
				},
				alpha = {
					type = "range",
					name = L["Frame alpha"],
					desc = L["Frame alpha"],
					order = 50,
					min = 0.0,
					softMin = 0.25,
					max = 1.0,
					step = 0.05,
					isPercent = true,
					arg = "wlrAnchorAlpha",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				size = {
					type = "range",
					name = L["Frame size"],
					desc = L["Frame size"],
					order = 60,
					min = 0.01,
					softMin = 0.25,
					max = 3,
					softMax = 2,
					step = 0.01,
					isPercent = true,
					arg = "wlrAnchorSize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
				timeleftheader = {
					type = "header",
					order = 69,
					name = L["Show time left"],
				},
				timeleft = {
					type = "toggle",
					name = L["Show time left"],
					desc = L["Show time left"],
					order = 70,
					arg = "showTimeleft",
					get = "getProfileOption",
					set = "setProfileOption",
					width = "full",
				},
				font = {
					type = 'select',
					dialogControl = 'LSM30_Font',
					name = L["Font"],
					desc = L["Font"],
					values = AceGUIWidgetLSMlists.font,
					arg = "timeleftFont",
					get = "getProfileOption",
					set = "setProfileOption",
					order = 80,
				},
				fontsize = {
					type = "range",
					name = L["Font size"],
					desc = L["Font size"],
					order = 90,
					min = 0.00,
					softMin = 7,
					max = 32,
					step = 1,
					arg = "timeleftFontsize",
					get = "getProfileOption",
					set = "setProfileOption",
				},
			},
		}, -- END output
    },
}
-- END Options

function WarlockReminder:OnProfileChanged(event, database, newProfileKey)
	wlrDB = self.db.profile
	self:CustomSetup()
end

function WarlockReminder:OnProfileReset(event, database, newProfileKey)
	wlrDB = self.db.profile
	self:CreateDefaultTables()
	self:Unlock()
end

function WarlockReminder:OnNewProfile(event, database, newProfileKey)
	wlrDB = self.db.profile
	self:CreateDefaultTables()
	self:Unlock()
end

-- Setup options
function WarlockReminder:SetUpOptions()
	local wlrprofile = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)
	-- Add dual-spec support
	local LibDualSpec = LibStub("LibDualSpec-1.0", true)
	if(LibDualSpec) then
		LibDualSpec:EnhanceDatabase(self.db, "WarlockReminder")
		LibDualSpec:EnhanceOptions(wlrprofile, self.db)
	end
	
	local config = LibStub("AceConfig-3.0")
	config:RegisterOptionsTable("WarlockReminder", wlroptions, {"wlr", "warlockreminder"})
	
	local registry = LibStub("AceConfigRegistry-3.0")
	-- Add option windows
   registry:RegisterOptionsTable("WarlockReminder Main", wlroptions)
	registry:RegisterOptionsTable("WarlockReminder Profiles", wlrprofile)
	
	local dialog = LibStub("AceConfigDialog-3.0")
	self.optionFrames = {
		main = dialog:AddToBlizOptions("WarlockReminder Main", "WarlockReminder"),
		profile = dialog:AddToBlizOptions("WarlockReminder Profiles", L["Profiles"], "WarlockReminder"),
	}
end

-- Get Function
function WarlockReminder:getProfileOption(info)
	return wlrDB[info.arg]
end

-- Set Function
function WarlockReminder:setProfileOption(info, value)
	wlrDB[info.arg] = value
	if(info.uiType == "cmd") then
		WarlockReminder:Print(L["|cffffff7f%s|r is now set to |cffffff7f[%s]|r"]:format(info.option.name, tostring(value)))
	end
	self:CustomSetup()
end

-- Color picker
function WarlockReminder:getProfileOptionColor(info)
	local c = wlrDB[info.arg]
	return c.r, c.g, c.b, c.a
end

function WarlockReminder:setProfileOptionColor(info, r, g, b, a)
	wlrDB[info.arg] = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
	self:CustomSetup()
end

